@@ -1,5 +1,4 @@
 from io import BytesIO
-import sys
 import textwrap
 import email
 import time
@@ -32,6 +31,7 @@ from typing import (
     AsyncIterable,
 )
 from typing_extensions import (
+    TYPE_CHECKING,
     Literal,
     NoReturn,
     TypedDict,
@@ -39,78 +39,25 @@ from typing_extensions import (
     Never,
 )
 
+if TYPE_CHECKING:
+    from urllib.parse import ParseResult
 
-# The precedence of HTTP libraries is
-# - Urlfetch (this is provided by Google App Engine, so if it's present you probably want it)
-# - Requests (popular library, the top priority for all environments outside Google App Engine, but not always present)
-# - Pycurl (another library, not always present, not as preferred as Requests but at least it verifies SSL certs)
-# - urllib2 with a warning (basically always present, fallback if needed)
-try:
-    import urllib.request as urllibrequest
-    import urllib.error as urlliberror
-except ImportError:
-    # Try to load in urllib2, but don't sweat it if it's not available.
-    pass
-
-try:
-    import pycurl  # pyright: ignore
-except ImportError:
-    pycurl = None
-
-try:
-    import httpx
-    import anyio
-    from httpx import Timeout as HTTPXTimeout
-    from httpx import Client as HTTPXClientType
-except ImportError:
-    httpx = None
-    anyio = None
-
-try:
-    import aiohttp
-    from aiohttp import ClientTimeout as AIOHTTPTimeout
-    from aiohttp import StreamReader as AIOHTTPStreamReader
-except ImportError:
-    aiohttp = None
-
-try:
-    import requests
-    from requests import Session as RequestsSession
-except ImportError:
-    requests = None
-else:
     try:
-        # Require version 0.8.8, but don't want to depend on distutils
-        version: str
-        version = requests.__version__
-        major: int
-        minor: int
-        patch: int
-        major, minor, patch = [int(i) for i in version.split(".")]
-    except Exception:
-        # Probably some new-fangled version, so it should support verify
+        from requests import Session as RequestsSession
+    except ImportError:
         pass
-    else:
-        if (major, minor, patch) < (0, 8, 8):
-            sys.stderr.write(
-                "Warning: the Stripe library requires that your Python "
-                '"requests" library be newer than version 0.8.8, but your '
-                '"requests" library is version %s. Stripe will fall back to '
-                "an alternate HTTP library so everything should work. We "
-                'recommend upgrading your "requests" library. If you have any '
-                "questions, please contact support@stripe.com. (HINT: running "
-                '"pip install -U requests" should upgrade your requests '
-                "library to the latest version.)" % (version,)
-            )
-            requests = None
 
-try:
-    from google.appengine.api import urlfetch  # pyright: ignore
-except ImportError:
-    urlfetch = None
+    try:
+        from httpx import Timeout as HTTPXTimeout
+        from httpx import Client as HTTPXClientType
+    except ImportError:
+        pass
 
-# proxy support for the pycurl client
-from urllib.parse import urlparse, ParseResult
+    try:
+        from aiohttp import ClientTimeout as AIOHTTPTimeout
+        from aiohttp import StreamReader as AIOHTTPStreamReader
+    except ImportError:
+        pass
 
 
 def _now_ms():
@@ -118,27 +65,61 @@ def _now_ms():
 
 
 def new_default_http_client(*args: Any, **kwargs: Any) -> "HTTPClient":
-    if urlfetch:
-        impl = UrlFetchClient
-    elif requests:
-        impl = RequestsClient
-    elif pycurl:
-        impl = PycurlClient
-    else:
-        impl = Urllib2Client
+    """
+    This method creates and returns a new HTTPClient based on what libraries are available. It uses the following precedence rules:
 
-    return impl(*args, **kwargs)
+    1. Urlfetch (this is provided by Google App Engine, so if it's present you probably want it)
+    2. Requests (popular library, the top priority for all environments outside Google App Engine, but not always present)
+    3. Pycurl (another library, not always present, not as preferred as Requests but at least it verifies SSL certs)
+    4. urllib with a warning (basically always present, a reasonable final default)
+
+    For performance, it only imports what it's actually going to use. But, it re-calculates every time its called, so probably save its result instead of calling it multiple times.
+    """
+    try:
+        from google.appengine.api import urlfetch  # type: ignore # noqa: F401
+    except ImportError:
+        pass
+    else:
+        return UrlFetchClient(*args, **kwargs)
+
+    try:
+        import requests  # noqa: F401
+    except ImportError:
+        pass
+    else:
+        return RequestsClient(*args, **kwargs)
+
+    try:
+        import pycurl  # type: ignore # noqa: F401
+    except ImportError:
+        pass
+    else:
+        return PycurlClient(*args, **kwargs)
+
+    return Urllib2Client(*args, **kwargs)
 
 
 def new_http_client_async_fallback(*args: Any, **kwargs: Any) -> "HTTPClient":
-    if httpx:
-        impl = HTTPXClient
-    elif aiohttp:
-        impl = AIOHTTPClient
-    else:
-        impl = NoImportFoundAsyncClient
+    """
+    Similar to `new_default_http_client` above, this returns a client that can handle async HTTP requests, if available.
+    """
 
-    return impl(*args, **kwargs)
+    try:
+        import httpx  # noqa: F401
+        import anyio  # noqa: F401
+    except ImportError:
+        pass
+    else:
+        return HTTPXClient(*args, **kwargs)
+
+    try:
+        import aiohttp  # noqa: F401
+    except ImportError:
+        pass
+    else:
+        return AIOHTTPClient(*args, **kwargs)
+
+    return NoImportFoundAsyncClient(*args, **kwargs)
 
 
 class HTTPClient(object):
@@ -159,6 +140,7 @@ class HTTPClient(object):
         verify_ssl_certs: bool = True,
         proxy: Optional[Union[str, _Proxy]] = None,
         async_fallback_client: Optional["HTTPClient"] = None,
+        _lib=None,  # used for internal unit testing
     ):
         self._verify_ssl_certs = verify_ssl_certs
         if proxy:
@@ -600,6 +582,7 @@ class RequestsClient(HTTPClient):
         verify_ssl_certs: bool = True,
         proxy: Optional[Union[str, HTTPClient._Proxy]] = None,
         async_fallback_client: Optional[HTTPClient] = None,
+        _lib=None,  # used for internal unit testing
         **kwargs,
     ):
         super(RequestsClient, self).__init__(
@@ -610,8 +593,12 @@ class RequestsClient(HTTPClient):
         self._session = session
         self._timeout = timeout
 
-        assert requests is not None
-        self.requests = requests
+        if _lib is None:
+            import requests
+
+            _lib = requests
+
+        self.requests = _lib
 
     def request(
         self,
@@ -785,6 +772,7 @@ class UrlFetchClient(HTTPClient):
         proxy: Optional[HTTPClient._Proxy] = None,
         deadline: int = 55,
         async_fallback_client: Optional[HTTPClient] = None,
+        _lib=None,  # used for internal unit testing
     ):
         super(UrlFetchClient, self).__init__(
             verify_ssl_certs=verify_ssl_certs,
@@ -806,8 +794,12 @@ class UrlFetchClient(HTTPClient):
         # to 55 seconds to allow for a slow Stripe
         self._deadline = deadline
 
-        assert urlfetch is not None
-        self.urlfetch = urlfetch
+        if _lib is None:
+            from google.appengine.api import urlfetch  # pyright: ignore
+
+            _lib = urlfetch
+
+        self.urlfetch = _lib
 
     def request(
         self, method: str, url: str, headers: Mapping[str, str], post_data=None
@@ -904,14 +896,14 @@ class UrlFetchClient(HTTPClient):
 
 
 class _Proxy(TypedDict):
-    http: Optional[ParseResult]
-    https: Optional[ParseResult]
+    http: Optional["ParseResult"]
+    https: Optional["ParseResult"]
 
 
 class PycurlClient(HTTPClient):
     class _ParsedProxy(TypedDict, total=False):
-        http: Optional[ParseResult]
-        https: Optional[ParseResult]
+        http: Optional["ParseResult"]
+        https: Optional["ParseResult"]
 
     name = "pycurl"
     _parsed_proxy: Optional[_ParsedProxy]
@@ -921,6 +913,7 @@ class PycurlClient(HTTPClient):
         verify_ssl_certs: bool = True,
         proxy: Optional[HTTPClient._Proxy] = None,
         async_fallback_client: Optional[HTTPClient] = None,
+        _lib=None,  # used for internal unit testing
     ):
         super(PycurlClient, self).__init__(
             verify_ssl_certs=verify_ssl_certs,
@@ -928,15 +921,21 @@ class PycurlClient(HTTPClient):
             async_fallback_client=async_fallback_client,
         )
 
-        assert pycurl is not None
-        self.pycurl = pycurl
+        if _lib is None:
+            import pycurl  # pyright: ignore[reportMissingModuleSource]
+
+            _lib = pycurl
+
+        self.pycurl = _lib
         # Initialize this within the object so that we can reuse connections.
-        self._curl = pycurl.Curl()
+        self._curl = _lib.Curl()
 
         self._parsed_proxy = {}
         # need to urlparse the proxy, since PyCurl
         # consumes the proxy url in small pieces
         if self._proxy:
+            from urllib.parse import urlparse
+
             proxy_ = self._proxy
             for scheme, value in proxy_.items():
                 # In general, TypedDict.items() gives you (key: str, value: object)
@@ -1091,7 +1090,7 @@ class PycurlClient(HTTPClient):
         msg = textwrap.fill(msg) + "\n\n(Network error: " + e.args[1] + ")"
         raise APIConnectionError(msg, should_retry=should_retry) from e
 
-    def _get_proxy(self, url) -> Optional[ParseResult]:
+    def _get_proxy(self, url) -> Optional["ParseResult"]:
         if self._parsed_proxy:
             proxy = self._parsed_proxy
             scheme = url.split(":")[0] if url else None
@@ -1103,6 +1102,7 @@ class PycurlClient(HTTPClient):
         pass
 
 
+# todo(major): rename this, urllib2 isn't called that anymore
 class Urllib2Client(HTTPClient):
     name = "urllib.request"
 
@@ -1111,21 +1111,33 @@ class Urllib2Client(HTTPClient):
         verify_ssl_certs: bool = True,
         proxy: Optional[HTTPClient._Proxy] = None,
         async_fallback_client: Optional[HTTPClient] = None,
+        _lib=None,  # used for internal unit testing
     ):
         super(Urllib2Client, self).__init__(
             verify_ssl_certs=verify_ssl_certs,
             proxy=proxy,
             async_fallback_client=async_fallback_client,
         )
+
+        if _lib is None:
+            import urllib.request as urllibrequest
+
+            _lib = urllibrequest
+        self.urllibrequest = _lib
+
+        import urllib.error as urlliberror
+
+        self.urlliberror = urlliberror
+
         # prepare and cache proxy tied opener here
         self._opener = None
         if self._proxy:
             # We have to cast _Proxy to Dict[str, str] because pyright is not smart enough to
             # realize that all the value types are str.
-            proxy_handler = urllibrequest.ProxyHandler(
+            proxy_handler = self.urllibrequest.ProxyHandler(
                 cast(Dict[str, str], self._proxy)
             )
-            self._opener = urllibrequest.build_opener(proxy_handler)
+            self._opener = self.urllibrequest.build_opener(proxy_handler)
 
     def request(
         self, method: str, url: str, headers: Mapping[str, str], post_data=None
@@ -1172,7 +1184,7 @@ class Urllib2Client(HTTPClient):
         if isinstance(post_data, str):
             post_data = post_data.encode("utf-8")
 
-        req = urllibrequest.Request(
+        req = self.urllibrequest.Request(
             url, post_data, cast(MutableMapping[str, str], headers)
         )
 
@@ -1185,7 +1197,7 @@ class Urllib2Client(HTTPClient):
             response = (
                 self._opener.open(req)
                 if self._opener
-                else urllibrequest.urlopen(req)
+                else self.urllibrequest.urlopen(req)
             )
 
             if is_streaming:
@@ -1195,11 +1207,11 @@ class Urllib2Client(HTTPClient):
 
             rcode = response.code
             headers = dict(response.info())
-        except urlliberror.HTTPError as e:
+        except self.urlliberror.HTTPError as e:
             rcode = e.code
             rcontent = e.read()
             headers = dict(e.info())
-        except (urlliberror.URLError, ValueError) as e:
+        except (self.urlliberror.URLError, ValueError) as e:
             self._handle_request_error(e)
         lh = dict((k.lower(), v) for k, v in iter(dict(headers).items()))
         return rcontent, rcode, lh
@@ -1225,21 +1237,19 @@ class HTTPXClient(HTTPClient):
         self,
         timeout: Optional[Union[float, "HTTPXTimeout"]] = 80,
         allow_sync_methods=False,
+        _lib=None,  # used for internal unit testing
         **kwargs,
     ):
         super(HTTPXClient, self).__init__(**kwargs)
 
-        if httpx is None:
-            raise ImportError(
-                "Unexpected: tried to initialize HTTPXClient but the httpx module is not present."
-            )
+        if _lib is None:
+            import httpx
 
-        if anyio is None:
-            raise ImportError(
-                "Unexpected: tried to initialize HTTPXClient but the anyio module is not present."
-            )
+            _lib = httpx
+        self.httpx = _lib
+
+        import anyio
 
-        self.httpx = httpx
         self.anyio = anyio
 
         kwargs = {}
@@ -1250,10 +1260,10 @@ class HTTPXClient(HTTPClient):
         else:
             kwargs["verify"] = False
 
-        self._client_async = httpx.AsyncClient(**kwargs)
+        self._client_async = self.httpx.AsyncClient(**kwargs)
         self._client = None
         if allow_sync_methods:
-            self._client = httpx.Client(**kwargs)
+            self._client = self.httpx.Client(**kwargs)
         self._timeout = timeout
 
     def sleep_async(self, secs):
@@ -1385,33 +1395,40 @@ class AIOHTTPClient(HTTPClient):
     name = "aiohttp"
 
     def __init__(
-        self, timeout: Optional[Union[float, "AIOHTTPTimeout"]] = 80, **kwargs
+        self,
+        timeout: Optional[Union[float, "AIOHTTPTimeout"]] = 80,
+        _lib=None,  # used for internal unit testing
+        **kwargs,
     ):
         super(AIOHTTPClient, self).__init__(**kwargs)
 
-        if aiohttp is None:
-            raise ImportError(
-                "Unexpected: tried to initialize AIOHTTPClient but the aiohttp module is not present."
-            )
+        if _lib is None:
+            import aiohttp
+
+            _lib = aiohttp
+
+        self.aiohttp = _lib
 
         self._timeout = timeout
         self._cached_session = None
 
     @property
     def _session(self):
-        assert aiohttp is not None
-
         if self._cached_session is None:
             kwargs = {}
             if self._verify_ssl_certs:
                 ssl_context = ssl.create_default_context(
                     cafile=stripe.ca_bundle_path
                 )
-                kwargs["connector"] = aiohttp.TCPConnector(ssl=ssl_context)
+                kwargs["connector"] = self.aiohttp.TCPConnector(
+                    ssl=ssl_context
+                )
             else:
-                kwargs["connector"] = aiohttp.TCPConnector(verify_ssl=False)
+                kwargs["connector"] = self.aiohttp.TCPConnector(
+                    verify_ssl=False
+                )
 
-            self._cached_session = aiohttp.ClientSession(**kwargs)
+            self._cached_session = self.aiohttp.ClientSession(**kwargs)
 
         return self._cached_session
 
